/* 

File:    samg.c
Purpose: Create an Simple Audio Manager (AUDIO_PP.IFO)

dvda-author  - Author a DVD-Audio DVD

(C) Dave Chapman <dave@dchapman.com> 2005

The latest version can be found at http://dvd-audio.sourceforge.net

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "audio.h"

int create_samg(char* audiotsdir,int titleset,fileinfo_t* files, int* ntracks, int ngroups, int* atsi_sectors, int startsector) {
  int i,j,g;
  int sectoroffset;
  FILE* fpout;
  char outfile[560];
  uint8_t samg[2048*8];

  memset(samg,0,sizeof(samg));
  memcpy(&samg[0],"DVDAUDIOSAPP",12);
  j=0;
  for (g=0;g<ngroups;g++) { j+=ntracks[g]; }
  uint16_copy(&samg[12],j);
  uint16_copy(&samg[14],0x0012);
  
  i=16;
  sectoroffset=startsector+64+3+3+atsi_sectors[0]; // sizeof(AUDIO_PP.IFO)+sizeof(AUDIO_TS.IFO)+sizeof(AUDIO_TS.BUP)+sizeof(ATS_01_1.IFO)
  for(g=0;g<ngroups;g++) {
    for(j=0;j<ntracks[g];j++) {
      i+=2;
      samg[i]=g+1; // Group number???
      i++;
      samg[i]=j+1;
      i++;
      uint32_copy(&samg[i],files[99*g+j].first_PTS);
      i+=4;
      uint32_copy(&samg[i],files[99*g+j].PTS_length);
      i+=4;
      i+=4;
      if (j==0) { samg[i]=0xc8; } else { samg[i]=0x48; }
      i++;
      switch(files[99*g+j].bitspersample) {
        case 16: samg[i]=0x0f; break;
        case 20: samg[i]=0x1f; break;
        case 24: samg[i]=0x2f; break;
        default: break;
      }
      i++;
      switch(files[99*g+j].samplerate) {
        case 48000:  samg[i]=0x0f; break;
        case 96000:  samg[i]=0x1f; break;
        case 192000: samg[i]=0x2f; break;
        case 44100:  samg[i]=0x8f; break;
        case 88200:  samg[i]=0x9f; break;
        case 176400: samg[i]=0xaf; break;
        default: break;
      }
      i++;

      switch(files[99*g+j].channels) {
        case 1: samg[i]=0; break;
        case 2: samg[i]=1; break;
        default:
          fprintf(stderr,"ERR: Unsupported number of channels (%d)\n",files[99*g+j].channels);
          exit(0);
      }
      i++;

      i+=20;
      uint32_copy(&samg[i],sectoroffset+files[99*g+j].first_sector);
      i+=4;
      uint32_copy(&samg[i],sectoroffset+files[99*g+j].first_sector);
      i+=4;
      uint32_copy(&samg[i],sectoroffset+files[99*g+j].last_sector);
      i+=4;
    }
    if (g < (ngroups-1)) {
      sectoroffset+=(files[99*g+ntracks[g]-1].last_sector+1)+atsi_sectors[g]+atsi_sectors[g+1];
    }
  }
  
  snprintf(outfile,sizeof(outfile),"%s/AUDIO_PP.IFO",audiotsdir);
  fpout=fopen(outfile,"wb+");
  for (i=0;i<8;i++) { fwrite(samg,1,sizeof(samg),fpout); }
  fclose(fpout);

  return(0);
}
